@@ -7,8 +7,9 @@ use DateTime::Format::Natural;
 
 use constant LANG_DEFAULT => 'en';
 
-my $debug = 0;
-my $lang  = LANG_DEFAULT;
+my $debug               = 0;
+my $format;
+my $lang                = LANG_DEFAULT;
 my @supported_languages = qw(de en);
 my %valid_languages     = map { $_ => 1 } @supported_languages;
 
@@ -18,23 +19,30 @@ sub parse_switches {
     use Getopt::Long qw(:config no_auto_abbrev no_ignore_case);
 
     my %opts;
-    GetOptions(\%opts,'d|debug','h|help','l|lang=s', 's|supported', 'V|version') or usage();
+    GetOptions(\%opts, qw(d|debug
+                          f|format=s
+                          h|help
+                          l|lang=s
+                          s|supported
+                          V|version)) or usage();
 
     usage()     if $opts{h};
     version()   if $opts{V};
     supported() if $opts{s};
 
-    $debug = $opts{d};
-    $lang  = $opts{l};
+    $debug  ||= $opts{d};
+    $lang   ||= $opts{l};
+    $format ||= $opts{f};
 }
 
 sub usage {
     print <<USAGE;
 Usage: $0 [switches]
   -d, --debug            debugging mode (experimental)
+  -f, --format           format of numeric dates
   -h, --help             help screen
   -l, --language code    language (country code)
-  -s, --supported	 list of supported languages
+  -s, --supported        list of supported languages
   -V, --version          print version
 USAGE
     exit;
@@ -55,7 +63,7 @@ unless ($valid_languages{$lang}) {
     $lang = LANG_DEFAULT;
 }
 
-my $parse = DateTime::Format::Natural->new(lang => $lang);
+my $parse = DateTime::Format::Natural->new(lang => $lang, format => $format);
 
 while (1) {
     print 'Input date string: ';
